import os, sys

# ISOHeader
__doc__ = """
Python interface to ISO9660 volume attributes (title, publisher, etc.)

I wrote this because the traditional Python ISO9660 package no longer
compiled very well with the new SWIG.  It seemed like a mess to work
with and this project took only about a half hour.  Additionally, this
is pure Python, and I couldn't think of a good reason to use C/SWIG for
a task this simple.  I used the file isosize.c (included with IsoInfo)
to get descriptions of the attributes.
See the class ISOHeader for usage.

This also works as a command line utility:

    python ISOHeader.py /dev/cdrom
"""

__author__ = 'David McClosky (dmcc@bigasterisk.com)'
__credits__ = """Based on the famous Bash script (as seen in the
CD-Writing-HOWTO) which does the same thing and cdrecord's isoinfo.c,
as seen in the Python package IsoInfo.  Thanks to Drew Perttula for
his code audit and suggestions."""
__version__ = "1.1"
__all__ = ['ISOHeader']

class ISOHeader:
    """Allows access to many of the attributes on an ISO9660 volume.

    Common attributes:
        volume_id (the CD title, also aliased to title)
        publisher_id
        creation_time (a string like '2002011313145500')
        data_preparer_id

    Example:
        cdrom = ISOHeader('/dev/cdrom')
        print cdrom.title
        print cdrom.publisher_id
        
    See the attributes static method for a list of all ISO attributes."""

    #           offset, len, description
    offsets = ((32767, 1,   'type'),
               (32768, 7,   'id'), 
               (32773, 1,   'version'), 
               (32776, 32,  'system_id'), 
               (32808, 32,  'volume_id'), 
               (32887, 3,   'volume_set_size'),
               (32891, 3,   'volume_seq_size'),
               (32958, 128, 'volume_set_id'), 
               (33086, 128, 'publisher_id'), 
               (33214, 128, 'data_preparer_id'), 
               (33342, 128, 'application_id'), 
               (33470, 32,  'copyright_file_id'), 
               (33506, 32,  'abstract_file_id'),
               (33581, 16,  'creation_time'), 
               (33598, 16,  'modification_time'), 
               (33615, 16,  'expiry_time'), 
               (33632, 16,  'effective_time'))

    def __init__(self, device, autostrip=1):
        """device is a device with a CD in it, like '/dev/hdd' or
        '/dev/cdrom'.  autostrip is whether each attribute should have
        whitespace stripped from each end."""
        dev = open(device)
        for offset, length, desc in self.offsets:
            dev.seek(offset)
            data = dev.read(length)
            if autostrip:
                data = data.strip()
            setattr(self, desc, data)
        dev.close()

        self.title = self.volume_id
    def attributes():
        """Returns a list of all ISOHeader attribute names"""
        return [tup[2] for tup in ISOHeader.offsets]
    attributes = staticmethod(attributes)

if __name__ == "__main__":
    device = sys.argv[1]
    iso = ISOHeader(device)

    for attr in ISOHeader.attributes():
        print attr, getattr(iso, attr)
