#!/usr/bin/python

"""
Point2Play Source Code
Copyright 2003 TransGaming Technologies Inc.
All rights reserved
Redistribution of Point2Play is forbidden without the expressed written 
permission of TransGaming Technologies Inc.
"""

import os, mount, log, system_detection

# related exceptions
class cdromError(Exception):
    """base class for cdrom errors"""

class notCDorDVDFstabEntryError(cdromError):
    """thrown when the given fstab line is not a cdrom or dvd drive"""

    def __init__(self, fstabline):
        self.fstabline = fstabline

    def __str__(self):
        return `self.fstabline`


class OpticalDrive: 
    """ This is really an abstract base class. We use it for deriving
        fstab-linked optical drives and dbus-detected optical drives. """

    def __init__(self, device_path, mount_path):
        self.device_path = device_path
        self.mount_path = mount_path
        # convert any escaped characters such as space ( the four-char string "\040" )
        self.mount_path = self.mount_path.decode('string_escape')

    def getDev(self):
        return self.device_path

    def getMountPath(self):
        return self.mount_path

    def isCDDeviceReadable(self):
        """checks if the current user has permissions to read the cd device"""
        return (os.access (self.getDev(), os.R_OK) )

    def isCDDeviceWriteable(self):
        """checks if the current user has permissions to write to the cd device"""
        return (os.access (self.getDev(), os.W_OK) )

    def isDeviceExists(self):
        """checks if the current user has permissions to write to the cd device"""
        return (os.access (self.getDev(), os.F_OK) )

    def isDriveMounted(self):
        """ Returns true if the drive is mounted. We do this by searching 
            /proc/mounts for the device-path (which should not change).
            This subroutine also makes sure the mount-path is kept up-to-date,
            as this can change with dbus systems. """

        fp = file( "/proc/mounts" )
        lines = fp.readlines()
        log.logger.info("Opened /proc/mounts...")
        for i in lines:
            (dev, path, rest ) = i.split( " ", 2 )
            log.logger.info("Looking for our device, " + self.device_path + "... Line contains device: " + dev)
            if dev == self.device_path:
                log.logger.info("Found the device " + dev + " in /proc/mounts.")
                # convert escaped characters
                path = path.decode('string_escape')
                if path != self.mount_path:
                    # the mount-path appears to have changed - update it in the object
                    log.logger.info("The mount path for " + dev + " seems to have changed from " + self.mount_path + " to " + path + ". Updating.")
                    self.mount_path = path
                return True
        return False


class fstabCdrom(OpticalDrive):
    """An optical disk device item linked to a line in the fstab file."""

    def __init__(self, fstabline):
        if not self.passFstabLine(fstabline):
            raise notCDorDVDFstabEntryError(fstabline)

    def passFstabLine(self, line):
        """returns true if the line passed in as 'line'
        is probably a cd or dvd drive
        sets the member variables device_path, mount_path, mount_type and options_dev
        to their required parts (optoins_dev comes from the dev= option).

        this MUST give the exact same output as the winex cdrom test does
        wine/files/drive.c:DRIVE_ProbeCD"""
        # hence the ugly nature of this code
        items = line.split()
        if len( items ) <= 2 :
            return False

        # device
        if (items[0][0] != '/' and items[0][0:4] != 'none'):
            return False
        self.device_path = items[0]
        # mount point
        if (items[1][0] != '/'):
            return False
        self.mount_path = items[1]
        if (self.mount_path[-1] == '/'):
            self.mount_path = self.mount_path[:-1]
        # convert escaped characters
        self.mount_path = self.mount_path.decode('string_escape')
        # type
        self.mount_type = items[2]
        # mount options (dev= part)
        self.options_dev = ''
        if len(items) > 3:
           dev_loc = items[3].find('dev=');
           if (dev_loc >= 0):
              # We need to advance the location to skip by the dev= crunk	
              dev_loc = dev_loc + 4 
              end_dev_loc = items[3].find(',', dev_loc)
              self.options_dev = items[3][dev_loc:end_dev_loc]

        # now for the test
        if (str.find(self.mount_type, "iso9660") == -1 and
          str.find(self.mount_type, "udf") == -1 and
          self.mount_type != "auto" and
          self.mount_type != "supermount" and
          self.mount_type != "subfs"):
            return False
        iscddvd = False
        if (self.device_path.find('cdr') >= 0): iscddvd = True
        elif (self.device_path.find('scd') >= 0): iscddvd = True
        elif (self.device_path.find('dvd') >= 0): iscddvd = True
        elif (self.mount_path.find('cdr') >= 0): iscddvd = True
        elif (self.mount_path.find('dvd') >= 0): iscddvd = True
        return iscddvd

class dbusOpticalDrive(OpticalDrive):
    """An optical disk device item NOT linked to a line in fstab, but detected through dbus."""
    def __init__(self, device_path, mount_path):
        OpticalDrive.__init__(self, device_path, mount_path)

    # yes, this is the entire class. 


def getDbusOpticalDrivesList():
    """ Uses dbus to detect optical drives. Return a list of dbusOpticalDrive objects. """

    drives = []

    try:
        import dbus
    except ImportError:
        log.logger.warning(_("Optical drive detection: the python dbus bindings are not installed."))
        return drives

    try:
        bus = dbus.SystemBus()
    except:
        log.logger.warning(_("Optical drive detection: dbus is either not running or is not installed."))
        return drives

    try:
        try:
            hal_manager_obj = bus.get_object("org.freedesktop.Hal", "/org/freedesktop/Hal/Manager")
            hal_manager = dbus.Interface(hal_manager_obj, "org.freedesktop.Hal.Manager")
            have_dbus_new = 1
        except:
            hal_service = bus.get_service ('org.freedesktop.Hal')
            hal_manager = hal_service.get_object ('/org/freedesktop/Hal/Manager','org.freedesktop.Hal.Manager')
            have_dbus_new = 0
    except:
        log.logger.warning(_("Optical drive detection: there appears to be an incompatibility with installed dbus bindings."))
        return drives

    try:  
        storage_udi_list = hal_manager.FindDeviceByCapability ('storage')
        volume_udi_list = hal_manager.FindDeviceByCapability ('volume')
    except:
        log.logger.warning(_("Optical drive detection: Hal does not appear to be installed."))
        return drives 


    for storage_udi in storage_udi_list:
        if have_dbus_new:
            storage_udi_obj = bus.get_object("org.freedesktop.Hal", storage_udi)
            device_file = storage_udi_obj.GetProperty('block.device',dbus_interface="org.freedesktop.Hal.Device")
            device_name = storage_udi_obj.GetProperty('info.product',dbus_interface="org.freedesktop.Hal.Device")
            device_type = storage_udi_obj.GetProperty('storage.drive_type',dbus_interface="org.freedesktop.Hal.Device")
        else:
            storage_udi_obj = hal_service.get_object(storage_udi, 'org.freedesktop.Hal.Device')
            device_file = storage_udi_obj.GetProperty('block.device')
            device_name = storage_udi_obj.GetProperty('info.product')
            device_type = storage_udi_obj.GetProperty('storage.drive_type')
       
        has_volume = ""
        if device_type == 'cdrom':
            for volume_udi in volume_udi_list:
                if have_dbus_new:
                    volume_udi_obj = bus.get_object("org.freedesktop.Hal", volume_udi)
                    parent = volume_udi_obj.GetProperty('info.parent',dbus_interface="org.freedesktop.Hal.Device")
                else:
                    volume_udi_obj = hal_service.get_object(volume_udi, 'org.freedesktop.Hal.Device')
                    parent = volume_udi_obj.GetProperty('info.parent')
             
                if parent == storage_udi:
                    if have_dbus_new:
                        volume_mount_point = volume_udi_obj.GetProperty('volume.mount_point',dbus_interface="org.freedesktop.Hal.Device")
                        volume_label = volume_udi_obj.GetProperty('volume.label',dbus_interface="org.freedesktop.Hal.Device")
                        has_volume = device_name
                    else:
                        volume_mount_point = volume_udi_obj.GetProperty('volume.mount_point')
                        volume_label = volume_udi_obj.GetProperty('volume.label')
                        has_volume = device_name
                
                    log.logger.info("Detected optical drive with device name=" + device_name + ", device file=" + device_file + ", mount point=" + volume_mount_point + ", volume label=" + volume_label)

                    newOpticalDrive = dbusOpticalDrive(device_file, volume_mount_point)
                    drives.append(newOpticalDrive)
            if has_volume != device_name:
                log.logger.info("Detected optical drive with device name=" + device_name + ", device file=" + device_file)
                # FIXME - is it ok to have an empty string for the mount point?
                newOpticalDrive = dbusOpticalDrive(device_file, "") 
                drives.append(newOpticalDrive)

    if not drives:
        log.logger.warning(_("Optical drive detection: No optical drives found - dbus/hal may not be correctly configured."))

    return drives


def getFstabOpticalDrivesList():
    """ Returns a list of fstabCdrom objects."""
    drives_list = []

    try:
        f = open('/etc/fstab')
    except IOError:
        log.logger.info("No fstab found (/etc/fstab).")
        return drives_list # just an empty list
    for line in f.readlines():
        try:
            newCdrom = fstabCdrom(line)
            drives_list.append(newCdrom)
        except notCDorDVDFstabEntryError, e:
            pass

    return drives_list

def getOpticalDrivesList():
    """ Returns a list of all findable optical drives. 
        First it looks for fstab-linked drives, then merges in the dbus-detected drives,
        eliminating duplicates based on device-path (e.g., /dev/hdc). """

    drives_list = getFstabOpticalDrivesList()
    try:
        drives_list.extend(getDbusOpticalDrivesList()) # tack on the dbus-detected drives
    except: # in case there's something wrong with the dbus libraries...
        pass

    dictionary = {} # this will be keyed on device so we get rid of dupes
    final_list = []

    for driveObject in drives_list:
        device_path = driveObject.getDev()
        if not dictionary.has_key(device_path):
            dictionary[device_path] = driveObject
            final_list.append(driveObject)

    return final_list
