"""
Point2Play Source Code
Copyright 2004, 2006 by TransGaming Technologies Inc.
All rights reserved
Redistribution of Point2Play is forbidden without the expressed written 
permission of TransGaming Technologies Inc.

"""

import pygtk
pygtk.require('2.0')
import gtk
import gtk.glade, gtk.gdk
import os, p2p_diag, log, Point2Play

# Some handy constants - for the return values from this popup
DEBUG_PLAY   = 1
DEBUG_CANCEL = 2

p2ppath = os.getenv("P2PPATH")
GLADE_FILE = p2ppath + "/gui/" + __name__ + ".glade"

# Default debug options that are available to the user.
# This gets tacked on to the end of each user's personal debug_channels list
default_debug_options = [ "+ddraw,+x11drv,+dib,+bitmap,+seh,+tid",
                          "+ole,+seh,+typelib,+tid",
                          "+relay,+seh,+tid",
                          "+server,+seh,+process,+thread,+tid",
                          "+all"
                        ]

def get_index_of_debug_option(debug_option, option_list):
    """ Given a debug option, find out its index in the given list option_list. """

    i = 0
    for current_opt in option_list:
        if debug_option == current_opt:
            return i
        i = i + 1
    return None
	
class WidgetsWrapper:
    def __init__(self):
        self.widgets = gtk.glade.XML(GLADE_FILE, None, "p2p_localized_msg")

    # Gives us the ability to do: widgets['widget_name'].action()
    def __getitem__(self, key):
        return self.widgets.get_widget(key)

class DebugProgramDialog:
    def __init__( self, Point2Play, standalone=1):
        """ When the user selects the Debug option from the main menu, this dialog
            shows a Cancel and Play button (standalone = 1). This dialog is also used
            in the Install dialog, as a way of viewing or setting the debug options.
            In that case (standalone=0), we show a Cancel and Continue button. """

        self.widgets = WidgetsWrapper()
        self.widgets.widgets.signal_autoconnect(self)
        self.Point2Play = Point2Play
        
        output_file = self.Point2Play.config["debugger_last_logfile"]
        if self.Point2Play.config["debugger_output_to_console"] == "True":
            self.widgets["output_to_console_checkbutton"].set_active(gtk.TRUE)
        else:
            self.widgets["output_to_console_checkbutton"].set_active(gtk.FALSE)

        if output_file:
            self.widgets["logfile_filename_textbox"].set_text(output_file)
            self.widgets["browse_button"].set_sensitive(gtk.TRUE)
            if self.Point2Play.config["debugger_use_logfile"]:
                self.widgets["output_to_logfile_checkbutton"].set_active(gtk.TRUE)
            else:
                self.widgets["output_to_logfile_checkbutton"].set_active(gtk.FALSE)
        else:
            self.widgets["logfile_filename_textbox"].set_sensitive(gtk.FALSE)
            self.widgets["browse_button"].set_sensitive(gtk.FALSE)
            self.widgets["output_to_logfile_checkbutton"].set_active(gtk.FALSE)

        # populate the combobox
        self.all_options = [""] + self.Point2Play.debug_options + default_debug_options
        for debug_option in self.all_options:
            log.logger.info("Appending option to debug options list: " + debug_option)
            self.widgets["debug_option_combobox"].append_text(debug_option)

        if not standalone:
           # Make the label for Play be "Continue".
           self.widgets["play_button"].set_label(_("Continue"))
           image = gtk.Image()
           image.set_from_stock(gtk.STOCK_GO_FORWARD, gtk.ICON_SIZE_BUTTON)
           self.widgets["play_button"].set_image(image)

        last_channel_used = self.Point2Play.config["debugger_last_channel"]
        if last_channel_used:
            self.widgets["debug_option_combobox"].set_active(get_index_of_debug_option(last_channel_used, self.all_options))


    def logfilecheck_cb (self, button):
        """ If we're outputting to a file, ungrey-out the checkbutton for output to the console,
            so the user can decide whether to split the output or not. No point to debugging
            if the output goes nowhere. So output can go to the console alone or to a file
            or both, but not neither.

            Also we enable the text entry box for the logfilename only if the checkbox is 
            checked. """
                
        state = self.widgets["output_to_logfile_checkbutton"].get_active()
        self.widgets["logfile_filename_textbox"].set_sensitive(state)
        if state:
           self.widgets["output_to_console_checkbutton"].set_sensitive(gtk.TRUE)
           self.widgets["play_button"].set_sensitive(gtk.TRUE)
           self.widgets["browse_button"].set_sensitive(gtk.TRUE)
        else:
           self.widgets["output_to_console_checkbutton"].set_active(gtk.TRUE)
           self.widgets["output_to_console_checkbutton"].set_sensitive(gtk.FALSE)
           self.widgets["browse_button"].set_sensitive(gtk.FALSE)


    def sensitive_cb(self, button):
        """ Whenever someone types something into the logfile blank, we go here
            to see if the Play/Continue button should be sensitive or not. """
        sensitive = len(self.widgets["logfile_filename_textbox"].get_text()) > 0
        self.widgets["play_button"].set_sensitive(sensitive)

    def browse_cb(self, button):
        """ Browse for an output file for the debug trace. """
        
        file_diag = p2p_diag.P2PFileSelection(0)
        file_diag.set_title(_("Select a Debug Output File"))
        file_diag.set_resizable(1)
        current_filename = self.widgets["logfile_filename_textbox"].get_text()
        if len(current_filename):  # figure out a likely starting-point
            if os.path.dirname(current_filename):
                file_diag.set_current_folder(os.path.dirname(current_filename))
        else:
            try:
                file_diag.set_current_folder(os.getenv("HOME"))
            except:
                pass
        resp = file_diag.run()
        if resp == gtk.RESPONSE_OK:
            filename = file_diag.get_filename()
            self.widgets["logfile_filename_textbox"].set_text( filename )
        file_diag.destroy()


    def run (self):
        """ We have overriden run() so we can validate the output filename when the user
            clicks Play/Continue and still keep the dialog open if the output filename is
            bad.

            Also, store the configuration so we can use it again next time. """

        while 1:
            response = gtk.Dialog.run( self.widgets["debug_dialog"] )       
            log.logger.info("Response is " + str(response))
            if response == gtk.RESPONSE_CANCEL or response == gtk.RESPONSE_DELETE_EVENT:
                # OK.. nada! user canceled
                log.logger.info("Debug cancelled.")
                return response
            else:
                debug_channel = self.widgets["debug_option_combobox"].get_active_text()
                log.logger.info("Debug channel to be used is " + str(debug_channel))

                # Validate the output file name if given
                output_file = None
                output_to_file = False
                if self.widgets["output_to_logfile_checkbutton"].get_active():
                    output_to_file = True
                    output_file = self.widgets["logfile_filename_textbox"].get_text()
                    if output_file == "":
                        err_diag = p2p_diag.P2PMessageDialog( self.widgets["debug_dialog"], 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _("You need to specify a filename if you wish to write the debug output into a file.") )
                        err_diag.run()
                        err_diag.destroy()
                        continue
                    if output_file[0] == '~':
                        # translate ~ to $HOME
                        output_file = os.getenv("HOME") + output_file[1:]
                    elif output_file[0] != '/':
                        # we'll try to assume $HOME. Tack it on to the beginning.
                        output_file = os.getenv("HOME") + '/' + output_file
                    if os.path.isdir(output_file):
                        err_diag = p2p_diag.P2PMessageDialog( self.widgets["debug_dialog"], 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _("Please specify a filename (not a directory) for your debug output.") )
                        err_diag.run()
                        err_diag.destroy()
       	                continue
                    if os.access(output_file, os.F_OK):
                        yesno_diag = p2p_diag.P2PMessageDialog(self.widgets["debug_dialog"], 0, gtk.MESSAGE_INFO, gtk.BUTTONS_YES_NO, _("Do you wish to overwrite the file %s for your debug output?") % (output_file ))
                        resp = yesno_diag.run()
                        yesno_diag.destroy()
                        if resp == gtk.RESPONSE_NO:
                            continue
                    if not os.access(os.path.dirname(output_file), os.F_OK) or not os.access(os.path.dirname(output_file), os.W_OK):
                        err_diag = p2p_diag.P2PMessageDialog( self.widgets["debug_dialog"], 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _("Sorry, the directory %s either does not exist or is not writable. Please select a different path for the debug output filename.") % (os.path.dirname(output_file)))
                        err_diag.run()
                        err_diag.destroy()
       	                continue                                         
              
                # is this a new debug channel? if so, save it in the user's personal list
                if debug_channel != "" and debug_channel not in self.Point2Play.debug_options and debug_channel not in default_debug_options:
                    self.Point2Play.debug_options.insert(0, debug_channel)
                # store the other debug info
                self.Point2Play.config["debugger_use_logfile"] = self.widgets["output_to_logfile_checkbutton"].get_active() == gtk.TRUE
                self.Point2Play.config["debugger_last_logfile"] = output_file
                self.Point2Play.config["debugger_last_channel"] = debug_channel
                self.Point2Play.config["debugger_output_to_console"] = self.widgets["output_to_console_checkbutton"].get_active() == gtk.TRUE
                self.Point2Play.write()
                return response
