#!/usr/bin/env python

"""
Point2Play Source Code
Copyright (c) 2004 by TransGaming Technologies, Inc.
All rights reserved
Redistribution of Point2Play is forbidden without the expressed written 
permission of TransGaming Technologies Inc.

"""

import log, gtk, gobject
import options_gui
import config

# Below is a dictionary linking the widget names used in the glade files 
# with config file items. When adding something new to the dictionary, the
# format is: 
#   widget name (checkbox/text entry/radio) : [ section, default_value, type, config key]
# Whenever possible, the widget name should be the same as the config key.
# So in most cases the key in the dictionary will be "" to indicate this.
# Sometimes it is not possible to use the same name since widget names need to be 
# unique and config names need to be unique only within a given section. 
# Some examples: UseMMap and FullDuplex.

# NOTE: do not use this for special-treatment config items such as pthreads_forced
# or passwords. See how proxy_password is treated in tgaccount_gui and Point2Play.py for
# a model.

# Some handy subscripts into the information array are:
SECTION=0
DEFAULT_VALUE=1
TYPE=2
KEY=3

# Here are the section names to use:
TG_SECTION = "transgaming"
X11DRV_SECTION = "x11drv"
WINMM_SECTION = "WinMM"
WINEOSS_SECTION = "wineoss"
WINEALSA_SECTION = "winealsa"
VERSION_SECTION = "version"
D3DGL_SECTION = "d3dgl"
FONT_SECTION = "fonts"
WINE_SECTION = "wine"
WINESERVER_SECTION = "Wineserver"
SYSTEM_SECTION = "system"
OPENGL_SECTION = "opengl"
PTHREADS_SECTION = "pthreads"
# Here are the types to use
INT=0
BOOL=1
TEXT=2

# Here's the dictionary of possible gui items (widget names). 
# Remember to leave key = "" when the config key is the same as the widget name.
# Note force_pthreads is absent - it's handled separately

gui_config_items = {
 "bigexe" : [TG_SECTION, 0, BOOL, ""],
 "doscwd": [TG_SECTION, "", TEXT, ""],
 "winexversion": [TG_SECTION, "default", TEXT, ""],
 "renice_wineserver" : [TG_SECTION, 0, BOOL, ""],
 "debugmsg" : [TG_SECTION, "+opengl", TEXT, ""],
 "Windows" : [VERSION_SECTION, "win98", TEXT, ""],
 "Cedega" : [VERSION_SECTION, "", TEXT, ""],
 "Managed" : [X11DRV_SECTION, 1, BOOL, ""],
 "Desktop" : [X11DRV_SECTION, "No", TEXT, ""],
 "UseXVidMode" : [X11DRV_SECTION, 1, BOOL, ""],
 "UseXRandR" : [X11DRV_SECTION, 0, BOOL, ""],
 "DXGrab" : [X11DRV_SECTION, 1, BOOL, ""],
 "MozillaControl" : [WINE_SECTION, 1, BOOL, ""],
 "ShowFPS" : [X11DRV_SECTION, 0, BOOL, ""],
 "ShowMem" : [X11DRV_SECTION, 0, BOOL, ""],
 "VideoRam" : [X11DRV_SECTION, "", TEXT, ""],
 "AGPVertexRam" : [X11DRV_SECTION, "", TEXT, ""],
 "NV_VAR": [X11DRV_SECTION, 1, BOOL, ""],
 "ARB_VBO": [X11DRV_SECTION, 1, BOOL, ""],
 "Drivers" : [WINMM_SECTION, "", TEXT, ""],  # FIXME - where used?
 "dsp0" : [WINEOSS_SECTION, "/dev/dsp", TEXT, ""],
 "mixer0": [WINEOSS_SECTION, "/dev/mixer", TEXT, ""],
 "ossUseMMap" : [WINEOSS_SECTION, 0, BOOL, "UseMMap"],
 "ossFullDuplex" : [WINEOSS_SECTION, 0, BOOL, "FullDuplex"],
 "pcm0" : [WINEALSA_SECTION, "hw", TEXT, ""],
 "ctl0" : [WINEALSA_SECTION, "hw", TEXT, ""],
 "alsaUseMMap" : [WINEALSA_SECTION, 0, BOOL, "UseMMap"],
 "AnisotropicTextureFiltering" : [D3DGL_SECTION, 0, BOOL, ""],
 "VertexShaders" : [D3DGL_SECTION, 1, BOOL, ""],
 "PixelShaders" : [D3DGL_SECTION, 1, BOOL, ""],
 "PixelShadersLevel" : [D3DGL_SECTION, "1.3", TEXT, ""],
 "VertexShadersLevel" : [D3DGL_SECTION, "1.1", TEXT, ""],
 "GLSL" : [D3DGL_SECTION, 0, BOOL, ""],
 "ClipSpaceFix" : [D3DGL_SECTION, 1, BOOL, ""],
 "FixedProgram" : [D3DGL_SECTION, "Auto", TEXT, ""],
 "FragmentOffset" : [D3DGL_SECTION, "Auto", TEXT, ""],
 "RectangleTextures" : [D3DGL_SECTION, "Auto", TEXT, ""],
 "DynamicVBO" : [D3DGL_SECTION, 0, BOOL, ""],
 "IndexVBO" : [D3DGL_SECTION, 0, BOOL, ""],
 "FBO" : [D3DGL_SECTION, 0, BOOL, ""],
 "FloatTextures" : [D3DGL_SECTION, 1, BOOL, ""],
 "OcclusionQueries" : [D3DGL_SECTION, 1, BOOL, ""],
 "GLExtensionBuffer" : [OPENGL_SECTION, "", TEXT, ""],
 "FixedGLExtensionBuffer" : [OPENGL_SECTION, 0, BOOL, ""],
 "FreeType" : [FONT_SECTION, 1, BOOL, ""],
 "Browser" : [WINE_SECTION, "", TEXT, ""],
 "SHMWineserver": [WINESERVER_SECTION, 0, BOOL, ""],
 "Scheduler": [WINESERVER_SECTION, "Default", TEXT, ""],
 "proxy_check": [TG_SECTION, 0, INT, ""],
 "proxy_port": [TG_SECTION, 3128, INT, ""],
 "proxy_host" : [TG_SECTION, "", TEXT, ""],
 "proxy_username": [TG_SECTION, "", TEXT, ""],
 "alsa_radio": [TG_SECTION, 0, BOOL, ""],
 "oss_radio": [TG_SECTION, 1, BOOL, ""],
 "cpu": [SYSTEM_SECTION, "", TEXT, ""],
 "cpu_ghz": [SYSTEM_SECTION, "", TEXT, ""],
 "memory": [SYSTEM_SECTION, "", TEXT, ""],
 "videocard_manufacturer": [SYSTEM_SECTION, "", TEXT, ""],
 "videocard_type": [SYSTEM_SECTION, "", TEXT, ""],
 "videocard_ram": [SYSTEM_SECTION, "", TEXT, ""],
 "agp_aperture_size": [SYSTEM_SECTION, "", TEXT, ""],
 "videocard_driver_version": [SYSTEM_SECTION, "", TEXT, ""],
 "soundcard": [SYSTEM_SECTION, "", TEXT, ""],
 "soundcard_driver": [SYSTEM_SECTION, "", TEXT, ""],
 "machine_bitness" : [SYSTEM_SECTION, "32", TEXT, ""],
 "kernel": [SYSTEM_SECTION, "", TEXT, ""],
 "x_version": [SYSTEM_SECTION, "", TEXT, ""],
 "distro": [SYSTEM_SECTION, "", TEXT, ""],
 "PthreadsValue": [PTHREADS_SECTION, "default", TEXT, ""],
 "scheduler_value": [TG_SECTION, "Default", TEXT, ""]}


# here is a dictionary of pixel shaders to make available depending on the 
# engine credential. Add levels of credential as necessary. For now,
# 0 corresponds to anything less than 600, and 1 to 600 and up.
PRE6 = 0
POST6 = 1
pixel_shader_levels = { POST6: ["2.0", "1.4", "1.3", "1.2", "1.1"],
                       PRE6 : ["1.4", "1.3", "1.2", "1.1"] }

vertex_shader_levels = { POST6 : ["2.0", "1.1"],
                        PRE6 : ["1.1"] }


def set_widget_value(widget, value):    
    """ Helper function """
    if (isinstance(widget, gtk.CheckButton) or isinstance(widget, gtk.RadioButton)):
        widget.set_active(int(value))
    elif isinstance(widget, gtk.Entry):
        widget.set_text(str(value))
    elif isinstance(widget, gtk.ComboBox):
        pos = 0
        model = widget.get_model()
        log.logger.info("Searching for " + str(value) + " in combobox.")
        iter = model.get_iter_first()
        while iter:
            if value == model.get_value(iter, 0):
                widget.set_active_iter(iter)
                log.logger.info("Found a match for the combobox.")
                return
            iter = model.iter_next(iter)
    else:   
        log.logger.info("UNKNOWN OBJECT TYPE - can't set value") # FIXME do something?


def get_widget_value(widget, widget_name, is_for_gameconfig_file):
    if (isinstance(widget, gtk.CheckButton) or
        isinstance(widget, gtk.RadioButton)):
        if widget.get_active():
            if is_for_gameconfig_file:
                value = 'Y'
            else:
                value = 1
        else:
            if is_for_gameconfig_file:
                value = 'N'
            else:
                value = 0
    elif isinstance(widget, gtk.Entry):
        value = widget.get_text()
    elif isinstance(widget, gtk.ComboBox):
        iter = widget.get_active_iter()
        model = widget.get_model()
        value = model.get_value(iter, 0)
        log.logger.info("Value obtained from ComboBox is " + str(value))
    else:
        value = ""
        log.logger.info("WARNING!! Unknown type for " + widget_name)
    return value

def set_default_value(object_with_widgets, widget_name):
    """ Given an object that contains the widgets in a WidgetWrapper style (so
        that we can say object_with_widgets.widgets[widget_name],
        look up a default value in the gui config items list and set it
        appropriately depending on what sort of widget the entry is."""
    value_to_set = gui_config_items[widget_name][DEFAULT_VALUE]
    set_widget_value(object_with_widgets.widgets[widget_name], value_to_set)


def load_value_from_config_file(widget_name, config_file):
    """ Given a gui widget name and a config file object, return the value from the 
        config file or a default value. This is not for game config file objects. """

    info_list = gui_config_items[widget_name]
    section = info_list[SECTION]
    default_value = info_list[DEFAULT_VALUE]
    type = info_list[TYPE]
    key = info_list[KEY]
    if key == "":
        key = widget_name

    if type == BOOL:
        try:
            value = int(config_file.get(section, key))
        except:
            value = default_value
    elif type == TEXT:
        try:
            value = config_file.get(section, key)
        except:
            value = default_value
    else:  # INT
        try:
            value = int(config_file.get(section, key))
        except:
            value = default_value

    return value


def get_widgetlist_by_section(section):
    """ Given a section (use the indexes at top of this file), returns a list of
        widget names of gui config items that go in that section. """
    widget_names = []
    for w, info_list in gui_config_items.iteritems():
        if info_list[SECTION] == section:
            widget_names.append(w)
    return widget_names

def write_by_section(fp, config_object, section):
    """ Given a file pointer, a config object and a section, write the gui config items
        to the file. Please note: this function assumes that the section name has
        already been written to the file. Also, this is not for game config files. """

    widgets_list = get_widgetlist_by_section(section)
    for widget_name in widgets_list:
        info_list = gui_config_items[widget_name]
        type = info_list[TYPE]
        key = info_list[KEY]
        if key == "":
            key = widget_name

        if type == BOOL:   # usu. checkbuttons/radiobuttons
            if config_object[widget_name] == 1:
                try:
                    fp.write( key + "=1\n")
                except:
                    log.logger.warning("Couldn't write to .point2playrc")
            elif config_object[widget_name] == 0:
                try:
                    fp.write( key + "=0\n")
                except:
                    log.logger.warning("Couldn't write to .point2playrc")
            else:
                log.logger.info("Something's wrong - the checkbox bools are not being stored as INTs")
        elif type == INT:
            fp.write(key + "=" + str(config_object[widget_name]) + "\n")
        else:  # TEXT
            fp.write(key + "=" + config_object[widget_name] + "\n")

# game config file helper functions


def load_value_from_game_config_object(widget_name, game_config, executable):
    """ Given a gui widget name and a game config object, return the value from the 
        object or a default value. """

    info_list = gui_config_items[widget_name]
    section = info_list[SECTION]
    default_value = info_list[DEFAULT_VALUE]
    type = info_list[TYPE]
    key = info_list[KEY]
    if key == "":
        key = widget_name

    if type == BOOL:
        value = game_config.get_option(section, key, executable, 1)
        if value == 'Y':
            value = 1
        elif value == 'N':
            value = 0
        else:   # well, it has to be Y or N so we'll go with the default value otherwise
            value = default_value
    elif type == TEXT:
        value = game_config.get_option(section, key, executable, 1)
    else:  # INT
        try:
            value = int(game_config.get_option(section, key, executable, 1))
        except:
            value = default_value

    # Ok, since our default config files come with default values that are marked with a ";;", the current
    # config file parser doesn't pick them up the first time around ( it will the second time, since the
    # parser will write a duplicate, un-commented version of these options ).  For this first time, we will
    # use the value that is currently set as a default in the game options above.  This will most likely have
    # to change or be extended since I don't imagine that the above defaults are valid for every version of
    # Cedega ( e.g. Winex 3.0 might not be best starting off with PixelShaders 1.3, etc... )
    if value == None:
        value = default_value

    return value

def get_section(widget_name):
    return gui_config_items[widget_name][SECTION]

def get_key(widget_name):
    return gui_config_items[widget_name][KEY]

# Here are some functions to get the config file to exchange data with the data widgets.

def load_config_data_into_widgets(object_with_widgets, data_widgets_list, Point2Play):
    """ Given an object with a WidgetsWrapper-type of data member called widgets,
        a list of data widgets and a Point2Play object, load the data from Point2Play.config
        into the widgets with the same name. """

    for w in data_widgets_list:
        set_widget_value(object_with_widgets.widgets[w], Point2Play.config[w])

def save_widget_data(object_with_widgets, data_widgets_list, Point2Play):
    """ Given an object with a WidgetsWrapper-type of data member called widgets,
        a list of data widgets and a Point2Play object, save the data from the widgets
        into the config object of the Point2Play object. """

    for w in data_widgets_list:
        value = get_widget_value(object_with_widgets.widgets[w], w, 0)
        Point2Play.config[w] = value
        
    Point2Play.write()

def load_gameconfig_data_into_widgets(object_with_widgets, data_widgets_list, game_config, executable = None):
    """ Given an object with widgets, a list of data widget names, a game_config
        object (see config.py), and the name of the game executable, load the data
        into the widgets."""

    for w in data_widgets_list:
        value = load_value_from_game_config_object(w, game_config, executable)
        set_widget_value(object_with_widgets.widgets[w], value)

def save_gameconfig_data(object_with_widgets, data_widgets_list, game_config, profile, real_exe = None, Point2Play=None, game=None, exe=None, gddb_file=None, shortcut=None):
    """ Save the widget data named in data_widgets_list from the object with widgets into
        the game config file """


    for w in data_widgets_list:
        value = get_widget_value(object_with_widgets.widgets[w], w, 1)
        # Did we come across one of the rare repeating values?
        if get_key(w):
            game_config.set_option( get_section(w), get_key(w), value, real_exe )
        else:
            game_config.set_option( get_section(w), w, value, real_exe )

        # Also save the info in the alternative section if there is one 
        # This is for options whose settings are mirrored between multiple
        # sections  - FIXME
        # alternate_section = option.get("alt_sec")
        # if alternate_section and alternate_section != "":
        # game_config.set_option( alternate_section, option["name"], self.GameSettings[game_name][exe_name][option["section"] + ":" + option["name"]], real_exe )

    # deal with alsa/oss radio selection. If alsa, WinMM:Drivers has to be set
    # to "winealsa.drv" and if oss, "wineoss.drv".

    alsa_radio = get_widget_value(object_with_widgets.widgets["alsa_radio"], "alsa_radio", 0)
    if alsa_radio:
        log.logger.info("ALSA has been selected - set WinMM Drivers to winealsa.drv")
        game_config.set_option( "WinMM", "Drivers", "winealsa.drv", real_exe)
    else:
        log.logger.info("OSS has been selected - set WinMM Drivers to wineoss.drv")
        game_config.set_option( "WinMM", "Drivers", "wineoss.drv", real_exe)


    dconfig = config.config_file( Point2Play.get_profile_load_list(game, shortcut, 1, install_gddb=gddb_file) )

    import os, os.path
    profile_bn = os.path.basename( profile )
    if profile_bn != "install_config":
      if Point2Play != None and game != None and exe != None:
        profile = Point2Play.get_shortcut_delta_name( game, shortcut )
      else:
        dconfig = config.config_file( Point2Play.get_profile_load_list(game, shortcut, 2) )
        profile = Point2Play.P2PFOLDER + Point2Play.GLOBAL_DELTA_FILENAME
    else:
        load_list = Point2Play.get_profile_load_list(None, None, install_gddb=gddb_file)
        dconfig = config.config_file( load_list )
    game_config.write( profile, dconfig )
    

def setup_comboboxentry_completion(comboboxentry, combo_strings):
    """ Given a ComboBoxEntry widget and a list of strings with which to populate it (combo_strings),
    populate the ComboBox and attach an EntryCompletion to it. """
    
    if not isinstance(comboboxentry, gtk.ComboBoxEntry):
        log.logger.warning("Error - calling setup_comboboxentry_completion without a ComboBoxEntry widget.")
        return

    if not isinstance(combo_strings, list):
        log.logger.warning("Error - calling setup_comboboxentry_completion without a list to populate it.")
        return

    liststore = gtk.ListStore(gobject.TYPE_STRING)
    for s in combo_strings:
        liststore.append(row=[s])
    comboboxentry.set_model(liststore)
    comboboxentry.set_text_column(0)
    completion = gtk.EntryCompletion()
    completion.set_model(liststore)
    completion.set_minimum_key_length(1)
    completion.set_text_column(0)
    comboboxentry.child.set_completion(completion)

