"""
Point2Play Source Code
Copyright 2003,2004,2005,2006 TransGaming Technologies Inc.
All rights reserved
Redistribution of Point2Play is forbidden without the expressed written 
permission of TransGaming Technologies Inc.

"""

import re, log, urllib, urlparse
import p2p_gui, popen2

def is_URL(incoming_string):
    """ Returns true if the incoming string appears to be a URL
        (or at least the kind of URL we care about). """

    if incoming_string[0:7] == "http://" or incoming_string[0:8] == "https://" or incoming_string[0:7] == "file://":
        return 1
    else:
        return 0

def is_windows_path(path):
    """ Returns true if path is a windows path. """
    if re.search('[C-Zc-z]:/', path):
        return True
    else:
        return False

def split_up_commandline( cmdline ):
    """ Separate a command line into the basic units for serving to an exec*-type command. 
        The main significance of this function is that quoted entities remain intact (and quoteless) 
        otherwise, we are simply splitting on space. Escaped quotes are allowed too. 

        Example: given the following cmdline:
          "Foobar's list 'of \" things' to separate \"including Quux's \\\"quoted\\\" strings\" and 'the \\' like'"
        this function returns the following list:
          ["Foobar's", 'list', 'of " things', 'to', 'separate', 'including Quux\'s \\"quoted\\" strings', 'and', "the \\' like"]

    """

    token_start_index = 0
    token_end_index = 0
    in_quotes = False
    splitlist = []
    current_quote_type = ""
    just_reached_end_of_quotes = False
    while token_end_index < len(cmdline):
        if cmdline[token_end_index] == '"' or cmdline[token_end_index] == "'":
            if in_quotes and cmdline[token_end_index] == current_quote_type and cmdline[token_end_index-1] != '\\':
                just_reached_end_of_quotes = True
                in_quotes = False
            elif not in_quotes:
                if token_end_index > 0 and cmdline[token_end_index-1] == ' ' or token_end_index == 0:
                    in_quotes = True
                    current_quote_type = cmdline[token_end_index]
        if cmdline[token_end_index] == ' ' and not in_quotes:
            offset = 0
            if just_reached_end_of_quotes:  # leave off the quotes
                offset = 1
                just_reached_end_of_quotes = False
            splitlist.append(cmdline[token_start_index+offset:token_end_index-offset])
            token_start_index = token_end_index + 1
        token_end_index += 1
    if cmdline[-1] != ' ':
        offset = 0
        if cmdline[-1] == '"' or cmdline[-1] == "'":
            offset = 1
        splitlist.append(cmdline[token_start_index+offset:token_end_index-offset])
    return splitlist

def quote_url(url):
    """ Quote a URL. Using the urllib.quote() alone is not a good idea because
        colons in the protocol and server should not be converted to %3A like this:
        >>> print urllib.quote('http://foogames.com:8088/support/fooo:bar.html')
        http%3A//foogames.com%3A8088/support/fooo%3Abar.html

        (the final : -> %3A conversion is ok)

    """

    tup = urlparse.urlsplit(url)
    parts = list(tup)

    # parts[0] is the protocol, parts[1] is the server and possibly port.
    # We want to quote parts[2] through parts[4] in case there are
    # special characters.

    for i in range(2, 4):
        parts[i] = urllib.quote(parts[i])
    url = urlparse.urlunsplit(parts)
    return url

def backslashify_quotes(incoming_string):
    """ Add backslashed backslashes before double-quotes in a string
         that is to be quoted with double-quotes. """

    incoming_string = incoming_string.replace('"', '\\"')
    return incoming_string

def select_string_in_left_pane(model, path, iter, incoming_string):
    return select_string_in_treemodel(model, path, iter, incoming_string, p2p_gui.widgets["main_game_folder_view"], p2p_gui.GAME_FOLDER_NAME_COL)

def select_string_in_right_pane(model, path, iter, incoming_string):
    return select_string_in_treemodel(model, path, iter, incoming_string, p2p_gui.widgets["main_shortcut_view"], p2p_gui.SHORTCUT_NAME_COL)

def select_string_in_treemodel(model, path, iter, incoming_string, liststore, column):
    """ Helper function, to be used in a TreeModel foreach loop.
        Finds the incoming_string and selects it in the given liststore. """

    if model.get_value(iter, column) == incoming_string:
        game_sel = liststore.get_selection()
        # select that row
        game_sel.select_path(path)
        # stop the looping
        return True
    else:
        return False


def which_version_newer(AMajor, AMinor, AMinorminor, BMajor, BMinor, BMinorminor):
    """ Compares two versions, A and B. 
        Returns 1 if B is newer, 0 if A is newer, and -1 if they are the same. """

    # Check if they're the same
    if AMajor == BMajor and AMinor == BMinor and AMinorminor == BMinorminor:
       return -1

    if (BMajor > AMajor):
       return 1
    elif (BMajor == AMajor):
       if (BMinor > AMinor):
          return 1
       elif (BMinor == AMinor):   
          if (BMinorminor > AMinorminor):
             return 1
          else:
             return 0
       else:
          # A newer
          return 0
    else:
       # A newer
       return 0

def go_to_url(url, Point2Play, gui = None):
    """ Go to a URL - report errors to the console if there is no GUI. """

    p2ppath = Point2Play.P2PPATH
    url = quote_url(url)  

    browserpath = Point2Play.get_browser_path()
    if browserpath:
        browserpath = "-b " + browserpath

    log.logger.info("Running " + p2ppath + "browserlaunch " + browserpath + " -u " + url)
    try:
        popen2.Popen4(p2ppath + "browserlaunch " + browserpath + " -u " + url, 1 )
    except:
        if gui:
            gui.notify_user(_("Sorry, unable to launch browser."), gui.widgets["main"])
        else:
            log.logger.warning(_("Sorry, unable to launch browser."))

