#!/usr/bin/env python

"""
Point2Play Source Code
Copyright (c) 2005 by TransGaming Technologies, Inc.
All rights reserved
Redistribution of Point2Play is forbidden without the expressed written 
permission of TransGaming Technologies Inc.

"""

# This is where the main Point2Play GUI lives. We have a class
# called WidgetWrapper that creates the GUI and lets us access
# all named widgets from anywhere.

import pygtk
pygtk.require('2.0')
import gtk
import gtk.glade
import os, shutil, socket, sys, popen2, tempfile
import log
import ClientCookie, urllib
import gddb
import Point2Play

try:
    import p2p_diag, winex_get, cedega_version_parse, winex_get_gui, license
    import update_checker_gui, download_gui, install_cedega_engine
except:
   raise Point2Play.Point2PlayError( _("Unable to load GUI components") + "(%s)" % str( sys.exc_info()[1] ) )

import error_log

# string for all games
ALL_GAMES_STRING = _("--- All Games ---")

# some column names for shortcut lists (used in the right pane of the main screen and the main config screen)
SHORTCUT_ICON_COL = 0
SHORTCUT_NAME_COL = 1
SHORTCUT_GAME_FOLDER_COL = 2
SHORTCUT_DIR_COL  = 3
SHORTCUT_REALEXE_COL = 4

# column names for the left pane in the main screen
GAME_FOLDER_NAME_COL = 0
GAME_FOLDER_ACTIVITY_COL = 1  # "activity" here means that something is running

# game folder activity icon size
GFA_ICON_WIDTH  = 10
GFA_ICON_HEIGHT = 10

ICON_WIDTH  = 32
ICON_HEIGHT = 32

# here are some exit-status type return values for our all-in-one update
DOWNLOAD_COMPLETE = 0
DOWNLOAD_NOT_COMPLETE = 1

TEMPDOWNLOAD_DIR = "/tmp/tg_mozctl_tmp"

# here are some indexes (used when passing around lists of what needs to be updated)
MSCOREFONTS = 0
MOZCONTROL = 1
CEDEGA = 2
P2P = 3 # currently unused
GDDB = 4

P2P_VERSION_URL = "http://www.transgaming.com/point2play_latestversion.php"
GDDB_MANIFEST_URL = "http://downloads.transgaming.com/gddb/MANIFEST"

log.logger.info("Temp download dir: " + TEMPDOWNLOAD_DIR)

p2ppath = os.getenv("P2PPATH")
GLADE_FILE = p2ppath + "/gui/" + __name__ + ".glade"
	
class WidgetsWrapper:
    def __init__(self):
        self.widgets = gtk.glade.XML(GLADE_FILE, None, "p2p_localized_msg")

    # this function lets us do widgets['widget_name'].action()
    def __getitem__(self, key):
        return self.widgets.get_widget(key)


# The line below this comment is very special. The first time you import this file, 
# the main window will be constructed. Any subsequent importations of this file will
# do nothing but allow you access to the main window's widgets.
# So you can then do things like this:
#     import p2p_gui
#     ...
#     x.set_transient_for(p2p_gui.widgets["main"])   # make the main window the parent of a dialog
#     ...
#     p2p_gui.widgets["play_menuitem"].set_sensitive(0)   # grey out something in the main window
#
# Important: don't do this for a dialog - you need to create them as you go, run them 
# and then destroy them. Only the main window is persistent. So we only do this
# for the main window.

widgets = WidgetsWrapper()

error_string = error_log.ErrorString()
message_string = error_log.ErrorString()

# note - the following methods are defined outside any class definition
# so that the main window and the setup wizard can call them.

def some_downloads_and_no_errors(ret):
    """ ret is a list of exit statuses
    Possible values are DOWNLOAD_COMPLETE, DOWNLOAD_NOT_COMPLETE """

    ok = False
    for i in ret:
        log.logger.info("In some_downloads_and_no_errors with " + str(i))
        if i == DOWNLOAD_NOT_COMPLETE:
            return False  # one error means we shouldn't say everything was hunky-dory
        if i == DOWNLOAD_COMPLETE:
            ok = True
    return ok

def check_for_updates(caller_gui, calling_from_wizard):
    """ Check for what packages require updating, and update them. """

    error_string.clear()
    message_string.clear()

    if calling_from_wizard:
        parent = caller_gui.widgets["wizard"]
    else:
        parent = widgets["main"]

    response = check_internet_connection_to_TG(caller_gui.Point2Play, parent)
    if not response:
        return

    if not os.access(caller_gui.Point2Play.P2PFOLDER + "/.gddb/", os.F_OK):
        os.mkdir( caller_gui.Point2Play.P2PFOLDER + "/.gddb" )
    caller_gui.Point2Play.update_gddb( )


    # if we got here the log in worked. Now see what needs updating.
    # construct a list indexed in this order: MSCOREFONTS, MOZCONTROL, CEDEGA, P2P
    needs_updating_list = []

    updatable_gddb_files = gddb_update_list( caller_gui.Point2Play )

    needs_updating_list.append(mscorefonts_requires_downloading(caller_gui.Point2Play))
    needs_updating_list.append(mozcontrol_requires_updating(caller_gui.Point2Play))
    needs_updating_list.append(cedega_requires_updating(caller_gui.Point2Play))
    needs_updating_list.append(p2p_requires_updating(caller_gui.Point2Play))
    needs_updating_list.append(gddb_requires_updating(updatable_gddb_files))

    dlg = update_checker_gui.UpdateChecker(needs_updating_list)
    dlg.widgets["update_checker"].set_transient_for(parent)
    response = dlg.widgets["update_checker"].run()
    needs_updating_list = dlg.checkbuttons # user decides what to install and what not to install
    dlg.widgets["update_checker"].destroy()

    if not response:
        return

    if tuple(needs_updating_list) == (0,0,0,0):
        notify_user(_("There are no updates available.  Cedega is up to date."), parent)
        return

    # ret is a list of exit statuses. Possible values are:
    #   DOWNLOAD_COMPLETE, DOWNLOAD_NOT_COMPLETE

    ret = []
 
    if needs_updating_list[MSCOREFONTS]:
        # MS Core Fonts
        ret.append(download_ms_core_fonts(caller_gui, calling_from_wizard))
    if needs_updating_list[MOZCONTROL]:
        # MozControl
        ret.append(download_mozcontrol_package(caller_gui, calling_from_wizard))
    if needs_updating_list[CEDEGA]:
        # Cedega
        ret.append(get_latest_version(caller_gui, calling_from_wizard))
    if needs_updating_list[P2P]:
        # Point2Play
        message_string.add_error(_("The latest version of Cedega is available in the downloads section at www.transgaming.com\nPlease check the Cedega How-To for instructions on downloading and installing the latest version of Cedega."))
    if needs_updating_list[GDDB]:
        ret.append(get_latest_gddb(caller_gui, calling_from_wizard, updatable_gddb_files))

                    
    messages = message_string.stringify()
    if messages != "":
        messages = "\n\nMessages:\n\n" + messages

    if some_downloads_and_no_errors(ret):
        notify_user(_("Update of packages is now complete." + messages), parent)
    else:
        errors = error_string.stringify()
        if errors != "":
            errors = "\n\nErrors:\n\n" + errors
        notify_user(_("Finished checking for updates." + errors + messages), parent)

def notify_user(message, parent):
    """ Just a little wrapper around the info-type dialog, to save space. """
    info_dlg = p2p_diag.P2PMessageDialog( parent, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, message )
    info_dlg.run()
    info_dlg.destroy()    


def get_latest_version(caller_gui, calling_from_wizard):

    install_cedega_engine_helper = install_cedega_engine.InstallCedegaEngineHelper( caller_gui.Point2Play, caller_gui, calling_from_wizard )
    install_cedega_engine_helper.install_cedega_engine(install_cedega_engine.INSTALL_DOWNLOAD_CHECK_UPDATES)

    # Enable widgets now that there is a version installed
    widgets["install_menuitem"].set_sensitive( 1 )
    widgets["install_button"].set_sensitive( 1 )
                
    return


def add_error_to_error_string(err_string):
    error_string.add_error( err_string )


def add_message_to_message_string(msg_string):
    message_string.add_error( msg_string )


def get_latest_gddb(caller_gui, calling_from_wizard, gddb_updatable):
    if calling_from_wizard:
        parent = caller_gui.widgets["wizard"]
    else:
        parent = widgets["main"]
        
    download_dialog = download_gui.DownloadDialog(caller_gui, caller_gui.Point2Play,
                                                  GDDB, calling_from_wizard, gddb_list=gddb_updatable)
    download_dialog.widgets["download_dialog"].set_transient_for(parent)
    download_dialog.widgets["download_dialog"].run()
    exit_status = download_dialog.exit_status
    logfile = download_dialog.logfile
    download_dialog.widgets["download_dialog"].destroy()

    while gtk.events_pending():  # do I need this?
        gtk.main_iteration()
    
    return exit_status

def download_ms_core_fonts(caller_gui, calling_from_wizard):
    # Download the Microsoft core fonts using our tg-font-install helper script
    # available in a separate package (transgaming_fontinstaller package)
    # Prompts the user with the Microsoft EULA first before actually 
    # starting the download process

    log.logger.info("Checking to see if MS Core Fonts needs updating...")

    if calling_from_wizard:
        parent = caller_gui.widgets["wizard"]
    else:
        parent = widgets["main"]

    # Display the Microsoft EULA so that the User can agree to it

    EULA_filename = caller_gui.Point2Play.FONTINSTALLER_PATH + "MS_FONTS_EULA.txt"
    EULA_fp = file(EULA_filename, "r" )
    EULA_diag = license.LicenseDialog( EULA_fp )
    resp = EULA_diag.run()
    EULA_diag.destroy()
       
    if resp == license.LICENSE_DECLINE:
        message_string.add_error( _("MS CORE Fonts: License declined."))
        return DOWNLOAD_NOT_COMPLETE
       
    while gtk.events_pending():  # do I need this?
        gtk.main_iteration()

    WINEXGLOBALPREFIX = caller_gui.Point2Play.WINEXGLOBALPREFIX
    # If the user doesn't have a WINEXGLOBALPREFIX/Fonts, create it
    global_fonts_filename = os.path.expanduser(WINEXGLOBALPREFIX + "Fonts")
    if (os.access(global_fonts_filename, os.F_OK) != 1):
        try:
            os.makedirs(global_fonts_filename)
        except:
            error_string.add_error( _("MS CORE Fonts: Unable to create destination directory " + WINEXGLOBALPREFIX + "Fonts for storing Microsoft(R) Core Fonts."))
            return DOWNLOAD_NOT_COMPLETE
    
    download_dialog = download_gui.DownloadDialog(caller_gui, caller_gui.Point2Play,
                                                  MSCOREFONTS, calling_from_wizard)
    download_dialog.widgets["download_dialog"].set_transient_for(parent)
    download_dialog.widgets["download_dialog"].run()
    exit_status = download_dialog.exit_status
    logfile = download_dialog.logfile
    download_dialog.widgets["download_dialog"].destroy()

    while gtk.events_pending():  # do I need this?
        gtk.main_iteration()

    if exit_status == 1:
        error_string.add_error( _("MS CORE Fonts: There was an error running the TransGaming Technologies Microsoft Core Font Installer. Please check the log file ") + logfile )
        return DOWNLOAD_NOT_COMPLETE
    elif exit_status == 0:
        return DOWNLOAD_COMPLETE
    else:  # cancelled
        error_string.add_error(_("MS CORE Fonts: Download aborted by user."))
        return DOWNLOAD_NOT_COMPLETE

def download_mozcontrol_package(caller_gui, calling_from_wizard):
    # Download the MozControl Package using our tg-mozctl-install helper script
    # available in a separate package (mozctlinstaller package)
    # Prompts the user with the Mozilla EULA first before actually 
    # starting the download process

    if calling_from_wizard:
        parent = caller_gui.widgets["wizard"]
    else:
        parent = widgets["main"]

    # Display the EULA 
    EULA_filename = caller_gui.Point2Play.MOZCTLINSTALLER_PATH +  "LICENSE.txt"
    EULA_fp = file(EULA_filename, "r" )
    EULA_diag = license.LicenseDialog( EULA_fp )
    resp = EULA_diag.run()
    EULA_diag.destroy()
       
    if resp == license.LICENSE_DECLINE:
        message_string.add_error( _("MozControl: License declined."))
        return DOWNLOAD_NOT_COMPLETE

    while gtk.events_pending():  # do I need this?
        gtk.main_iteration()

    download_dialog = download_gui.DownloadDialog(caller_gui, caller_gui.Point2Play,
                                                  MOZCONTROL, calling_from_wizard)
    download_dialog.widgets["download_dialog"].set_transient_for(parent)
    download_dialog.widgets["download_dialog"].run()
    exit_status = download_dialog.exit_status
    logfile = download_dialog.logfile
    download_dialog.widgets["download_dialog"].destroy()

    if exit_status == 1:
        error_string.add_error( _("MozControl: There was an error running the script to obtain the MozControl package - please check the log file ") + logfile)
        return DOWNLOAD_NOT_COMPLETE
    elif exit_status == 0:
        return DOWNLOAD_COMPLETE
    else:  # cancelled
        error_string.add_error( _("MozControl: download aborted by user."))
        # delete the temporary download directory and its contents
        log.logger.info("Clearing out the temporary download directory for the MozControl helper script: " + TEMPDOWNLOAD_DIR)
        try:
            for root, dirs, files in os.walk(TEMPDOWNLOAD_DIR, topdown=False):
                for name in files:
                    os.remove(os.path.join(root, name))
                for name in dirs:
                    os.rmdir(os.path.join(root, name))
            os.rmdir(TEMPDOWNLOAD_DIR)
        except:
            log.logger.info(str( sys.exc_info()[1] ))
            log.logger.warning("Unable to remove MozControl's temporary download directory, " + TEMPDOWNLOAD_DIR)
        return DOWNLOAD_NOT_COMPLETE

def check_internet_connection_to_TG(Point2Play, parent):
    """ Pops up a dialog with the error and returns success or failure. """

    password = Point2Play.config["password"]
    Point2Play.setup_environment_for_proxy()
    try:
        winex_get.login_transgaming( Point2Play.config["username"], password )
    except socket.gaierror, e:
        err_diag = p2p_diag.P2PMessageDialog( parent, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _("Unable to connect to server\n%s") % e[1] )
        err_diag.run()
        err_diag.destroy()
        return False
    except:
        err_diag = p2p_diag.P2PMessageDialog( parent, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, "%s" % str( sys.exc_info()[1] ) )
        err_diag.run()
        err_diag.destroy()
        return False
    else:
        return True

    

def mozcontrol_requires_updating(Point2Play):
    """ Returns True/False if mozcontrol requires updating or not. """
    script = Point2Play.MOZCTLINSTALLER_PATH + "tg-mozctl-install"
    if os.path.exists(script):
        # make sure we can write to the tempdownload_dir if it exists - otherwise
        # pass down a unique temp dir in the environment variable TEMPDOWNLOAD_DIR.
        log.logger.info("MOZCONTROL is " + str(MOZCONTROL))
        log.logger.info("Checking to see if the MozControl temporary download directory exists: " + str(TEMPDOWNLOAD_DIR))
        if os.path.isdir(TEMPDOWNLOAD_DIR) and not os.access(TEMPDOWNLOAD_DIR, os.W_OK):
            # if I don't use globals() here, python seems to think I'm creating
            # a local, and then it complains about use before assignment in the
            # if statement, too. Duh.
            globals()["TEMPDOWNLOAD_DIR"] = tempfile.mkdtemp()

            os.environ["TEMPDOWNLOAD_DIR"] = TEMPDOWNLOAD_DIR
            log.logger.info("Making a temp directory called " + TEMPDOWNLOAD_DIR + " to pass to the mozctl installer helper script.")
        cmd = script + " -check-update"
        log.logger.info("Executing " + cmd)
        p = popen2.Popen4(cmd, 1)
        status = p.wait()
        exit_status = os.WEXITSTATUS(status)
        log.logger.info("In mozcontrol_requires_updating... exit status of " + str(exit_status))
        return exit_status
    else:
        message_string.add_error(_("Missing MozControl Installer Package.")) # FIXME
        return False # we cannot update without the installer anyway

def mscorefonts_requires_downloading(Point2Play):
    """ Check if the user has a complete set of MS Core Fonts. These are apparently
    very old and never change so once they have been downloaded they shouldn't need 
    to be updated - unless the user messes with them. So we check to see if the
    directory exists and all the font files."""
    
    font_files = [ "AndaleMo.TTF", "Arialbd.TTF", "Arialbi.TTF", "Ariali.TTF",
                   "Arial.TTF", "AriBlk.TTF", "Comicbd.TTF", "Comic.TTF", "courbd.ttf",
                   "courbi.ttf", "couri.ttf", "cour.ttf", "Georgiab.TTF", "Georgiai.TTF",
                   "Georgia.TTF", "Georgiaz.TTF", "Impact.TTF", "Timesbd.TTF", "Timesbi.TTF",
                   "Timesi.TTF", "Times.TTF", "Trebucbd.ttf", "trebucbi.ttf", "trebucit.ttf",
                   "trebuc.ttf", "Verdanab.TTF", "Verdanai.TTF", "Verdana.TTF", "Verdanaz.TTF",
                   "Webdings.TTF" ]

    if os.path.exists(Point2Play.WINEXGLOBALPREFIX + "Fonts"):
        for font_file in font_files:
            if not os.path.exists(Point2Play.WINEXGLOBALPREFIX + "/Fonts/" + font_file):
                log.logger.info("Incomplete set of MS Core Fonts: missing " + font_file)
                return True
    else:
        log.logger.info("No " + Point2Play.WINEXGLOBALPREFIX + "Fonts directory found")
        return True

    return False

def cedega_requires_updating(Point2Play):
    """ Check to see if Cedega requires updating.
        Only give an authoritative Yes if we can confirm we need an update. """

    password = Point2Play.config["password"]
    Point2Play.setup_environment_for_proxy()
    try:
        winex_get.login_transgaming( Point2Play.config["username"], password )
    except socket.gaierror, e:
        return False
    except:
        return False
    else:
        try:
            ver_list = winex_get.winex_versions()
        except socket.gaierror, e:
            return False
        except:
            return False
        if len( ver_list ):
            latest = ver_list[len( ver_list ) - 1]
            versionparse = cedega_version_parse.ParseVersion(latest["version"])
            latest["version"] = versionparse.get_parsed_version_string()

            for ver in Point2Play.versions:
                if ver["version"] == latest["version"]:
                    log.logger.info("Cedega does not require updating")
                    return False
    return True

def p2p_requires_updating(Point2Play):
    """ Check to see if Cedega requires updating """

    # Read the latest version  
    # Compare with the current version which should be in the file 

    Point2Play.setup_environment_for_proxy()
    try:
        fp = ClientCookie.urlopen(P2P_VERSION_URL)
        latest_version = fp.readline()
        log.logger.info("Latest version of Cedega is " + latest_version)
        fp.close()
    except:
        error_string.add_error(_("Can't seem to retrieve the file on transgaming.com containing the latest version number of Cedega."))
        return False

    if latest_version > Point2Play.VERSION_STRING:
        return True
    return False


def gddb_requires_updating( l ):
    if l == None or len(l) == 0:
        return False
        
    return True


def gddb_update_list(Point2Play, update_all_files=None):
    """ Check to see if GDDB requires updating """

    gddb_revisions = []
    list = []
    Point2Play.setup_environment_for_proxy()
    try:
        fp = ClientCookie.urlopen(GDDB_MANIFEST_URL)
        s = fp.readline()
        while s != None:
            s = s.strip()
            if s == None or s == "":
                break
            (fname, ver) = s.split( "\t", 1 )
            gddb_revisions.append( [fname, ver] )
            s = fp.readline()
        fp.close()
    except Exception, e:
        error_string.add_error(_("Can't seem to retrieve the file on transgaming.com containing the latest versions of GDDB."))
        return list

    for x in gddb_revisions:
        try:
            g = gddb.load_gddb( Point2Play.P2PFOLDER + "/.gddb/" + x[0], silent=1 )
            local_ver = -2^19
            remote_ver = float( x[1] )
            if g != None:
                local_ver = float( g["header"]["revision"] )
            
            if remote_ver > local_ver or update_all_files != None:
                list.append( x[0] )
        
        except Exception, e:
            pass

    return list
