#!/usr/bin/env python

"""
Point2Play Source Code
Copyright (c) 2004 by TransGaming Technologies, Inc.
All rights reserved
Redistribution of Point2Play is forbidden without the expressed written 
permission of TransGaming Technologies Inc.

"""

import log, cdrom
import os, re, sys, config, misc_utils

# these are optical drive letters to use. Note that G is left out on purpose - it is already 
# mapped by default.
optical_drive_letters = ['F', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z' ]


# If Point2Play is being launched by the packaged launch script, then it needs to be able
# to be launched from anywhere.  In this case, P2PPATH will already be set, and we can use
# that as a path to locate our files.
p2ppath = os.getenv("P2PPATH")
if p2ppath:
    AGPTEST_LOCATION = p2ppath + "/agp_test/agp_test"
    GLXINFO_LOCATION = p2ppath + "/tests/glxinfo"
    ST64BIT_LOCATION = p2ppath + "/systemtests/64bit"
else:
    AGPTEST_LOCATION = "./agp_test/agp_test"
    GLXINFO_LOCATION = "./tests/glxinfo"
    ST64BIT_LOCATION = "./systemtests/64bit"

# these are the widgets that we try to autodetect
system_data_widgets_list = ["cpu", "cpu_ghz", "memory", "videocard_manufacturer", "videocard_type",
     "videocard_ram", "agp_aperture_size", "videocard_driver_version", "soundcard", 
     "soundcard_driver", "machine_bitness", "kernel", "x_version", "distro"]

os.environ['PATH'] = "/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:/usr/bin/X11"
# Note: we assume grep, which, tail and awk are installed somewhere in this PATH

def round_to_nearest_power_of_two(number):
    """ Takes a number and approximates (upward) to the nearest power of two.
    example, given 503, this function will return 512."""
    # Warning! Given 513, it will return 1024. Is that right? FIXME

    approximation = 2
    while number > approximation:
        approximation = approximation * 2
    return approximation

def utility_exists(name):
    pipeFile = os.popen("which " + name)
    while pipeFile.read() != "": pass
    status = pipeFile.close()
    if status:
        log.logger.info("The utility '" + name + "' does not appear to be in any of the usual places.")
        return False
    else: 
        return True
    

def detect_cpu():
    if os.path.exists("/proc/cpuinfo"):
        pipeFile = os.popen("grep 'model name' /proc/cpuinfo | tail -n 1 | awk -F': ' '{print $2}'")
        cpu = pipeFile.readline()
        return cpu[:-1]
    return ""

def detect_cpu_ghz():
    if os.path.exists("/proc/cpuinfo"):
        # fixme - does /proc/cpuinfo *always* store this as MHz? 
        pipeFile = os.popen("grep 'MHz' /proc/cpuinfo | tail -n 1 | awk -F': ' '{print $2}'")
        cpu_mhz = pipeFile.readline()
        cpu_mhz = float(cpu_mhz[:-1])
        # now convert to ghz and round
        cpu_ghz = cpu_mhz / 1000
        return str(round(cpu_ghz, 2))
    return ""

def detect_memory():
    if os.path.exists("/proc/meminfo"):
        pipeFile = os.popen("grep MemTotal: /proc/meminfo | awk '{print $2}'")
        mem = pipeFile.readline()
        mem = float(mem[:-1])
        mem = mem / 1024
        return str(int(mem))
    if utility_exists("free"):
        pipeFile = os.popen("free | grep Mem | awk '{print $2}'")
        mem = pipeFile.readline()
        mem = float(mem[:-1])
        mem = mem / 1024
        return str(int(mem))
    return ""

def detect_videocard_manufacturer():
    if os.path.exists(GLXINFO_LOCATION):
        pipeFile = os.popen(GLXINFO_LOCATION + " | grep 'OpenGL vendor string' | awk -F': ' '{print $2}'")
        videocard_manufacturer = pipeFile.readline()
        return videocard_manufacturer[:-1]
    log.logger.warning(_("The glxinfo program seems to be missing from your Cedega package."))
    return ""

def detect_videocard_type():
    if os.path.exists(GLXINFO_LOCATION):
        pipeFile = os.popen(GLXINFO_LOCATION + " | grep 'OpenGL renderer string' | awk -F': ' '{print $2}'")
        cardtype = pipeFile.readline()
        return cardtype[:-1]
    log.logger.warning(_("The glxinfo program seems to be missing from your Cedega package."))
    return ""

def detect_videocard_ram():
    """ Searches for the VideoRAM line in various log files.
        Converts an assumed kByte amount to megs. FIXME - can I assume these are all expressed in K? """

    videoram = 0
    if os.path.exists("/var/log/XFree86.0.log"):
        pipeFile = os.popen("grep -s -i -h -e Memory: -e VideoRAM: /var/log/XFree86.0.log | awk -F: '{ print $3 }' | awk -F' ' '{ print $1 }'")
        videoram_line = pipeFile.readline()
        match = re.search('([0-9]+)',videoram_line)
        if match:
             videoram = match.group(1)
    if os.path.exists("/var/log/Xorg.0.log"):
        pipeFile = os.popen("grep -s -i -h -e Memory: -e VideoRAM: /var/log/Xorg.0.log | awk -F: '{ print $3 }' | awk -F' ' '{ print $1 }'")
        videoram_line = pipeFile.readline()
        status = pipeFile.close()
        if not status:
            match = re.search('([0-9]+)',videoram_line)
            if match:
                videoram = match.group(1)
    videoram = int(videoram)
    if videoram > 0:
        videoram = videoram / 1024
        videoram = round_to_nearest_power_of_two(videoram)
        return str(videoram)
    return ""

def detect_agp_aperture_size():
    """ Returns the agp aperture size."""

    if os.path.exists(AGPTEST_LOCATION):
        pipeFile = os.popen(AGPTEST_LOCATION)
        response = pipeFile.readline()
        match = re.search('N/A', response)
        if match:
            log.logger.info("In AGP Aperture Size Test: " + response)
            return "N/A"
        else:
            match = re.search('AGP Memory available:\s+(\d+) MB', response)
            if match:
                return match.group(1)
            else:
                return ""  # shouldn't happen
                log.logger.info("------------ Unexpected AGP Aperture Size Test output -----------------")
    log.logger.warning(_("The AGP Aperture Size Test seems to be missing from your Cedega package."))
    return ""

def detect_videocard_driver_version():
    if os.path.exists(GLXINFO_LOCATION):
        pipeFile = os.popen(GLXINFO_LOCATION + " | grep 'OpenGL version string' | awk -F': ' '{print $2}'")
        version = pipeFile.readline()
        return version[:-1]
    log.logger.warning(_("The glxinfo program seems to be missing from your Cedega package."))
    return ""

# This function returns whether or not OpenGL direct rendering is available.  This value is
# requested by the OpenGL Direct Rendering test.
def detect_opengl_direct_rendering():
    if os.path.exists(GLXINFO_LOCATION):
        pipeFile = os.popen(GLXINFO_LOCATION + " | grep 'direct rendering' | awk -F': ' '{print $2}'")
        version = pipeFile.readline()
        return version[:-1]
    log.logger.warning(_("The glxinfo program seems to be missing from your Cedega package."))
    return ""

# Add an addition check for alsa setups here, mainly to detect the only the first soundcard
# if there are more than one configured
def detect_soundcard():
    if os.path.exists("/proc/asound/cards"):
        pipeFile = os.popen("cat /proc/asound/cards | head -n2 | tail -n1")
        soundcard = pipeFile.readline().strip()
        pipeFile.close()
        return soundcard[:-1]		    
    if utility_exists("lspci"):
        pipeFile = os.popen("lspci -v |grep 'Multimedia audio controller' | awk -F': ' '{print $2}'")
        soundcard = pipeFile.readline()
        pipeFile.close()
        return soundcard[:-1]
    return ""

def detect_machine_bitness():
    """ Returns the string 64 if we're running a 64-bit distro, and "32" otherwise.
        This info is useful (for example) in the ALSA mmap test. """

    if os.path.exists (ST64BIT_LOCATION):
        ret = os.spawnl (os.P_WAIT, ST64BIT_LOCATION);
        if ret == 0:
            return "64"
        return "32"
    log.logger.warning (_("The 64bit test program seems to be missing from your Cedega package."))
    return "32"

def ensure_sbin_in_path():
    """ Make sure /usr/sbin and /sbin are both in the $PATH environment variable. """
    log.logger.info("In ensure_sbin_in_path")

    pathvar = os.environ["PATH"]
    log.logger.info("PATH is " + pathvar)
    if pathvar.find("/sbin:") == -1:
        # it has neither, so add them both
        log.logger.info("Adding both /usr/sbin and /sbin to the PATH")
        os.environ["PATH"] = pathvar + ":/usr/sbin:/sbin"
        return

    # if we got here, we have one or both.

    if pathvar.find("/usr/sbin:") == -1:
        # this means we have /sbin but not /usr/sbin
        log.logger.info("Adding /usr/sbin to the PATH")
        os.environ["PATH"] = pathvar + ":/usr/sbin"
        return

    # if we get here, we have /usr/sbin and we may or may not also have /sbin
    if pathvar[0:6] != "/sbin:" and pathvar.find(":/sbin:") == -1:
        log.logger.info("Adding /sbin to the PATH")
        os.environ["PATH"] = pathvar + ":/sbin"

    # if we got here, it means we had both, and that's fine. Do nothing.

def detect_soundcard_driver():
    """ Detect whether OSS or ALSA is used. Note: we cannot assume that all alsa modules
        are just compiled as modules, because with kernel 2.6.x it's possible to compile them
        into the kernel (this was a previous way of detecting).        
    """

    ensure_sbin_in_path()

    if os.path.exists("/proc/asound/cards"):
        if utility_exists("alsactl"):
            pipeFile = os.popen("alsactl -v | awk -F' ' '{ print $3 }'")
            version = pipeFile.readline()
            pipeFile.close()
            return "ALSA Version " + version[:-1]
        return "ALSA"
    return "OSS"

def detect_kernel():
    if utility_exists("uname"):
        pipeFile = os.popen("uname -r")
        kernel = pipeFile.readline()
        return kernel[:-1]
    return ""

def detect_x_version():
    if utility_exists("X"):
        os.environ["SHELL"] = "/bin/sh"  # make sure we're using sh for the redirection of stderr
        pipeFile = os.popen("X -version 2>&1 | grep -si 'XFree86 Version'")
        xver = pipeFile.readline()
        status = pipeFile.close()
        if status:  # didn't find XFree86 - try Xorg
            os.environ["SHELL"] = "/bin/sh" 
            pipeFile = os.popen("X -version 2>&1 | grep -si 'X Window System Version'")
            xver = pipeFile.readline()
            status = pipeFile.close()
            if status: # didn't find "X Window System Version"
                os.environ["SHELL"] = "/bin/sh"
                pipeFile = os.popen("X -version 2>&1 | grep -si 'Xorg Version'")
                xver = pipeFile.readline()
                status = pipeFile.close()
                if status: # didn't find "Xorg" at all
                    return ""
                return xver[:-1] # return string containing Xorg
            match = re.search('X Window System Version(.*)$', xver)
            return "Xorg Version" + match.group(1)
        return xver[:-1]   # the xfree version
    return ""

def get_firstline(filename):
    """ Returns the first line of a file. """

    pipeFile = os.popen("head -n 1 " + filename)
    contents = pipeFile.readline()
    status = pipeFile.close()
    if status:
        return ""
    return contents[:-1]

def detect_distro():

    # Note: order is important. Don't use a dictionary here. 
    # For example: if we detected the distro was Debian, we would never detect Xandros 
    # FIXME - perhaps we can figure out a way later to detect Ubuntu?

    distro_list = [ "Xandros", "Mandrake", "Red Hat", "Gentoo", "Slackware", "SuSE", "Linspire", "Lindows",
                    "Knoppix", "Arch", "Debian" ]
    file_list = [ "/etc/xandros-desktop-version", "/etc/mandrake-release", "/etc/redhat-release",
                  "/etc/gentoo-release", "/etc/slackware-version", "/etc/SuSE-release",
                  "/etc/linspire-version", "/etc/lindowsos-version", "/etc/knoppix-version",
                  "/etc/arch-release", "/etc/debian_version" ]

    if os.path.exists("/bin/lsb_release"):
    	pipeFile = os.popen("/bin/lsb_release -i | awk -F':' '{print $2}'")
	distributor_line = pipeFile.readline().strip()
	pipeFile.close()
	pipeFile = os.popen("/bin/lsb_release -r | awk -F':' '{print $2}'")
	release_line = pipeFile.readline().strip()
	pipeFile.close()
	pipeFile = os.popen("/bin/lsb_release -c | awk -F':' '{print $2}'")
	codename_line = pipeFile.readline().strip()
	pipeFile.close()
	return distributor_line + " " + release_line + " (" + codename_line + ")"
    else:
     	for distro, a_file in zip(distro_list, file_list):
            if os.path.exists(a_file):
               line = get_firstline(a_file)
               match = re.search('^Version: (.*)$', line)
               if match:
                  line = match.group(1)  # gets rid of "Version: "
               match = re.search(distro, line)
               if match:
                  return line
               else:
                  return distro + " " + line   # prefix the distro name when it isn't in the file


def detect_optical_drives(Point2Play, force=False):
    """ Stores optical drive info in the global configuration delta file. 
        Later this will all be incorporated into the GUI. For now it is just
        done behind the scenes. This function can be called from the setup wizard.

        If force=False, grep for Drive info in the global delta file,
        If force=True, always run dbus. 
    """

    fstabOpticalDriveList = cdrom.getFstabOpticalDrivesList()
    if fstabOpticalDriveList:
        log.logger.info("Optical drives found in /etc/fstab.")
        return  # this will all be handled properly by the engine

    dbusOpticalDrivesList = cdrom.getDbusOpticalDrivesList()
    if dbusOpticalDrivesList:
        profile = Point2Play.P2PFOLDER + Point2Play.GLOBAL_DELTA_FILENAME
        if not os.access(profile, os.F_OK): 
            log.logger.warning(_("No global delta file - something's wrong."))
            return

        if not force:
            # if we aren't supposed to force a re-detect, check to see if there's drive info
            # check if the string 'Drive ' occurs in the global delta file
            pipeFile = os.popen("grep -s -m 1 'Drive ' " + Point2Play.P2PFOLDER + Point2Play.GLOBAL_DELTA_FILENAME)
            drive = pipeFile.readline();
            exitstatus = pipeFile.close()
            if drive:
                return # found some drive info - don't force the re-detection

        gdconfig = config.config_file( Point2Play.get_profile_load_list(None, None) )
        drives_count = 0
        for drive in dbusOpticalDrivesList:
            drive_name = "Drive " + optical_drive_letters[drives_count]
            gdconfig.set_option( drive_name, "Type", "cdrom", None)
            gdconfig.set_option( drive_name, "Device", drive.getDev(), None)
            gdconfig.set_option( drive_name, "FailReadOnly", "Y", None)
            drives_count = drives_count + 1
        dconfig = config.config_file( Point2Play.get_profile_load_list(None, None, 2) )
        gdconfig.write(profile, dconfig)
    else:
        log.logger.warning(_("No optical drives have been found."))

# the setup wizard and Global Options gui use these keys

def make_detected_system_info_dictionary():
    """ Returns a dictionary of system information. Keys are
        listed in this module's "system_data_widgets_list" variable
        which is used by the Point2Play object to load and store 
        them in the user's cedegarc file. """

    system_info = {
        "cpu" : detect_cpu(),
        "cpu_ghz": detect_cpu_ghz(),
        "memory": detect_memory(),
        "videocard_manufacturer": detect_videocard_manufacturer(),
        "videocard_type": detect_videocard_type(),
        "videocard_ram": detect_videocard_ram(),
        "agp_aperture_size": detect_agp_aperture_size(),
        "videocard_driver_version": detect_videocard_driver_version(),
        "soundcard": detect_soundcard(),
        "soundcard_driver": detect_soundcard_driver(),
        "machine_bitness": detect_machine_bitness(),
        "kernel": detect_kernel(),
        "x_version": detect_x_version(),
        "distro" : detect_distro()
    }
    return system_info


def get_alsa_oss_info_from_cedegarc(Point2Play):
    """ This function will read the user's Point2Play object and
        figure out from the soundcard driver string ("OSS", "ALSA"
        or "ALSA Version ...") and machine bitness (32 or 64) what 
        the various settings for ALSA should be if applicable.
        It is used when storing the detected values 
        for the very first time in the .global.delta file,
        and also for reverting to default in the Global Settings."""
        
    dict = {}
    soundcard_driver_info = Point2Play.config["soundcard_driver"]
    if soundcard_driver_info[0:4] == "ALSA":
        dict["alsa_radio"] = 1
        dict["oss_radio"] = 0
        version = soundcard_driver_info[13:]
        bitness = Point2Play.config["machine_bitness"] # either "32" or "64"
        if bitness == "32":
            dict["alsaUseMMap"] = 1
        else: # 64 bit - means we need to check the ALSA version if there is one
            if not version:
                return dict # because there's not much more we can do

            log.logger.info("64 bit... check ALSA version now.")
            # compare the ALSA version with 1.0.11
            (major, minor, minorminor) = version.split(".", 2)
            if version[0:8] == "1.0.11rc":  # special case ... if it's an RC, unset mmap
                dict["alsaUseMMap"] = 0
                log.logger.info("ALSA version is a 1.0.11 RC - unset MMap.")
                return dict

            comp_value = misc_utils.which_version_newer(1, 0, 11, int(major), int(minor), int(minorminor) )
            if comp_value == 1: # alsa version is newer than 1.0.11
                dict["alsaUseMMap"] = 1
                log.logger.info("ALSA version newer than 1.0.11 - set MMap.")
            elif comp_value == 0:
                dict["alsaUseMMap"] = 0
                log.logger.info("ALSA version older than 1.0.11 - unset MMap.")
            else: # -1 means they're the same, oddly enough.
                dict["alsaUseMMap"] = 1
                log.logger.info("ALSA version is 1.0.11 but not an RC - set MMap.")
    else: # OSS
        dict["oss_radio"] = 1
        dict["alsa_radio"] = 0
    return dict
